@@ -1,22 +1,16 @@
-from StringIO import StringIO
-
 from zope.interface import implements, Interface
 from zope.component import adapts, getMultiAdapter, queryUtility
 
 from plone.i18n.normalizer.interfaces import IIDNormalizer
 from plone.portlets.interfaces import IPortletDataProvider
 from plone.app.portlets.portlets import base
-from plone.app.portlets import cache
 
 from zope import schema
 from zope.formlib import form
 
-from plone.memoize import ram
 from plone.memoize.instance import memoize
-from plone.memoize.compress import xhtml_compress
 
 from Acquisition import aq_inner, aq_base, aq_parent
-from AccessControl import getSecurityManager
 from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
 from Products.CMFCore.utils import getToolByName
 
@@ -48,13 +42,14 @@ class INavigationPortlet(IPortletDataProvider):
             required=False)
     
     root = schema.Choice(
-            title=_(u"label_navigation_root_path", default=u"Root path"),
+            title=_(u"label_navigation_root_path", default=u"Root node"),
             description=_(u'help_navigation_root',
-                          default=u"A folder to use for the base folder "
-                                   "where the navigation tree, sitemap, "
-                                   "breadcrumbs and tabs will be rooted. "),
+                          default=u"You may search for and choose a folder "
+                                    "to act as the root of the navigation tree. "
+                                    "Leave blank to use the Plone site root."),
             required=False,
-            source=SearchableTextSourceBinder({'is_folderish' : True}))
+            source=SearchableTextSourceBinder({'is_folderish' : True},
+                                              default_query='path:'))
                             
     includeTop = schema.Bool(
             title=_(u"label_include_top_node", default=u"Include top node"),
@@ -120,26 +115,6 @@ class Assignment(base.Assignment):
         self.topLevel = topLevel
         self.bottomLevel = bottomLevel
 
-def _render_cachekey(fun, self):
-    key = StringIO()
-    print >> key, getToolByName(aq_inner(self.context), 'portal_url')()
-    print >> key, cache.get_language(aq_inner(self.context), self.request)
-
-    catalog = getToolByName(self.context, 'portal_catalog')
-    counter = catalog.getCounter()
-    print >> key, counter
-
-    print >> key, aq_inner(self.context).getPhysicalPath()
-    
-    user = getSecurityManager().getUser()
-    roles = user.getRolesInContext(aq_inner(self.context))
-    print >> key, roles
-
-    print >> key, self.data._p_mtime
-    print >> key, self.properties._p_mtime
-
-    return key.getvalue()
-
 class Renderer(base.Renderer):
 
     def __init__(self, context, request, view, manager, data):
@@ -204,6 +179,9 @@ class Renderer(base.Renderer):
         
         rootPath = getRootPath(self.context, currentFolderOnly, topLevel, str(self.data.root))
         
+        if rootPath is None:
+            return rootPath
+        
         if rootPath == self.urltool.getPortalPath():
             return portal
         else:
@@ -227,9 +205,8 @@ class Renderer(base.Renderer):
     def update(self):
         pass
 
-    @ram.cache(_render_cachekey)
     def render(self):
-        return xhtml_compress(self._template())
+        return self._template()
 
     _template = ViewPageTemplateFile('navigation.pt')
     recurse = ViewPageTemplateFile('navigation_recurse.pt')
@@ -244,6 +221,7 @@ class AddForm(base.AddForm):
         return Assignment(name=data.get('name', u""),
                           root=data.get('root', u""),
                           currentFolderOnly=data.get('currentFolderOnly', False),
+                          includeTop=data.get('includeTop', False),
                           topLevel=data.get('topLevel', 0),
                           bottomLevel=data.get('bottomLevel', 0))
 
@@ -370,9 +348,13 @@ def getRootPath(context, currentFolderOnly, topLevel, root):
         contextPath = '/'.join(context.getPhysicalPath())
         if not contextPath.startswith(rootPath):
             return None
-        contextSubPathElements = contextPath[len(rootPath)+1:].split('/')
-        if len(contextSubPathElements) < topLevel:
+        contextSubPathElements = contextPath[len(rootPath)+1:]
+        if contextSubPathElements:
+            contextSubPathElements = contextSubPathElements.split('/')
+            if len(contextSubPathElements) < topLevel:
+                return None
+            rootPath = rootPath + '/' + '/'.join(contextSubPathElements[:topLevel])
+        else:
             return None
-        rootPath = rootPath + '/' + '/'.join(contextSubPathElements[:topLevel])
     
     return rootPath
